@@ -48,7 +48,8 @@ from keras.utils.np_utils import to_categorical
 
 from cnn2snn import load_quantized_model, convert
 
-from ..training import get_training_parser, compile_model, evaluate_model, print_history_stats
+from ..training import (get_training_parser, compile_model, evaluate_model, print_history_stats,
+                        RestoreBest)
 
 
 def get_data(data_file, input_size):
@@ -112,10 +113,18 @@ def train_model(model, train_data, train_labels, batch_size, epochs):
         batch_size (int): the batch size
         epochs (int): the number of epochs
     """
+    # Define training callbacks
+    callbacks = []
+
+    # Model checkpoints (save best model and retrieve it when training is complete)
+    restore_model = RestoreBest(model)
+    callbacks.append(restore_model)
+
     history = model.fit(train_data,
                         train_labels,
                         batch_size=batch_size,
-                        epochs=epochs)
+                        epochs=epochs,
+                        callbacks=callbacks)
     print_history_stats(history)
 
 
