@@ -21,7 +21,9 @@ Module used to compute mAP scores for YOLO classification.
 import keras
 import numpy as np
 
+from keras.utils import io_utils
 from .processing import load_image, preprocess_image, decode_output
+from .box_utils import compute_overlap
 
 
 class MapEvaluation(keras.callbacks.Callback):
@@ -42,6 +44,8 @@ class MapEvaluation(keras.callbacks.Callback):
              image
             is_keras_model (bool, optional): indicated if the model is a Keras
              model (True) or an Akida model (False)
+            decode_output_fn (Callable, optional): function to decode model's outputs.
+             Defaults to :func:`decode_output` (yolo decode output function).
 
         Returns:
             A dict mapping class names to mAP scores.
@@ -56,7 +60,8 @@ class MapEvaluation(keras.callbacks.Callback):
                  obj_threshold=0.5,
                  nms_threshold=0.5,
                  max_box_per_image=10,
-                 is_keras_model=True):
+                 is_keras_model=True,
+                 decode_output_fn=decode_output):
 
         super().__init__()
         self._model = model
@@ -70,6 +75,7 @@ class MapEvaluation(keras.callbacks.Callback):
         self._nms_threshold = nms_threshold
         self._max_box_per_image = max_box_per_image
         self._is_keras_model = is_keras_model
+        self._decode_output = decode_output_fn
 
     def on_epoch_end(self, epoch, logs=None):
         """ Keras callback called at the end of an epoch.
@@ -82,12 +88,15 @@ class MapEvaluation(keras.callbacks.Callback):
                 values of the Model’s metrics are returned.
                 Example: {‘loss’: 0.2, ‘acc’: 0.7}. Defaults to None.
         """
-        if epoch % self._period == 0 and self._period != 0:
+        epoch += 1
+        if self._period != 0 and (epoch % self._period == 0 or
+                                  epoch == self.params.get('epochs', -1)):
             _map, average_precisions = self.evaluate_map()
-            print('\n')
+            io_utils.print_msg("")
             for label, average_precision in average_precisions.items():
-                print(self._labels[label], '{:.4f}'.format(average_precision))
-            print('mAP: {:.4f}'.format(_map))
+                io_utils.print_msg(self._labels[label] + ' {:.4f}'.format(average_precision))
+            io_utils.print_msg('mAP: {:.4f}\n'.format(_map))
+            logs.update({'map': _map})
 
     def evaluate_map(self):
         """ Evaluates current mAP score on the model.
@@ -141,8 +150,8 @@ class MapEvaluation(keras.callbacks.Callback):
                 output = potentials.reshape(
                     (h, w, len(self._anchors), 4 + 1 + self._num_classes))
 
-            pred_boxes = decode_output(output, self._anchors, self._num_classes,
-                                       self._obj_threshold, self._nms_threshold)
+            pred_boxes = self._decode_output(output, self._anchors, self._num_classes,
+                                             self._obj_threshold, self._nms_threshold)
 
             score = np.array([box.get_score() for box in pred_boxes])
             pred_labels = np.array([box.get_label() for box in pred_boxes])
@@ -201,8 +210,8 @@ class MapEvaluation(keras.callbacks.Callback):
                         true_positives = np.append(true_positives, 0)
                         continue
 
-                    overlaps = self._compute_overlap(np.expand_dims(d, axis=0),
-                                                     annotations)
+                    overlaps = compute_overlap(np.expand_dims(d, axis=0), annotations,
+                                               mode="outer_product", box_format="xyxy")
                     assigned_annotation = np.argmax(overlaps, axis=1)
                     max_overlap = overlaps[0, assigned_annotation]
 
@@ -240,38 +249,6 @@ class MapEvaluation(keras.callbacks.Callback):
 
         return average_precisions
 
-    @staticmethod
-    def _compute_overlap(a, b):
-        """ Computes overlap between two boxes defined by their top left and
-        bottom right corner coordinates.
-        Code originally from https://github.com/rbgirshick/py-faster-rcnn.
-
-        Args:
-            a (ndarray): (N, 4) ndarray of float
-            b (ndarray): (K, 4) ndarray of float
-
-        Returns:
-            (N, K) ndarray of overlap between input boxes
-        """
-        area = (b[:, 2] - b[:, 0]) * (b[:, 3] - b[:, 1])
-
-        iw = np.minimum(np.expand_dims(a[:, 2], axis=1), b[:, 2]) - np.maximum(
-            np.expand_dims(a[:, 0], 1), b[:, 0])
-        ih = np.minimum(np.expand_dims(a[:, 3], axis=1), b[:, 3]) - np.maximum(
-            np.expand_dims(a[:, 1], 1), b[:, 1])
-
-        iw = np.maximum(iw, 0)
-        ih = np.maximum(ih, 0)
-
-        ua = np.expand_dims(
-            (a[:, 2] - a[:, 0]) * (a[:, 3] - a[:, 1]), axis=1) + area - iw * ih
-
-        ua = np.maximum(ua, np.finfo(float).eps)
-
-        intersection = iw * ih
-
-        return intersection / ua
-
     @staticmethod
     def _compute_ap(recall, precision):
         """ Compute the average precision, given the recall and precision
