@@ -26,7 +26,7 @@ from keras import Model
 
 from cnn2snn import load_quantized_model, quantize
 
-from ..layer_blocks import separable_conv_block
+from ..layer_blocks import yolo_head_block
 from ..imagenet.model_akidanet import akidanet_imagenet
 from ..utils import fetch_file
 
@@ -86,40 +86,7 @@ def yolo_base(input_shape=(224, 224, 3),
 
     # Add YOLO top layers to the base model
     input_shape = base_model.input_shape
-    x = base_model.layers[-1].output
-    x = separable_conv_block(x,
-                             filters=1024,
-                             name='1conv',
-                             kernel_size=(3, 3),
-                             padding='same',
-                             use_bias=False,
-                             add_activation=True,
-                             add_batchnorm=True)
-    x = separable_conv_block(x,
-                             filters=1024,
-                             name='2conv',
-                             kernel_size=(3, 3),
-                             padding='same',
-                             use_bias=False,
-                             add_activation=True,
-                             add_batchnorm=True)
-    x = separable_conv_block(x,
-                             filters=1024,
-                             name='3conv',
-                             kernel_size=(3, 3),
-                             padding='same',
-                             use_bias=False,
-                             add_activation=True,
-                             add_batchnorm=True)
-    x = separable_conv_block(x,
-                             filters=(nb_box * (4 + 1 + classes)),
-                             name='detection_layer',
-                             kernel_size=(3, 3),
-                             padding='same',
-                             use_bias=True,
-                             add_activation=False,
-                             add_batchnorm=False)
-
+    x = yolo_head_block(base_model.layers[-1].output, num_boxes=nb_box, classes=classes)
     model = Model(inputs=base_model.input, outputs=x, name='yolo_base')
 
     # Initialize detection layer weights
@@ -195,7 +162,7 @@ def yolo_voc_pretrained():
         anchors = pickle.load(handle)
 
     model_name = 'yolo_akidanet_voc_iq8_wq4_aq4.h5'
-    file_hash = '01f6ed4bc802ffb3cca71b5ba281925de1bb92bcd14aea562336366c6a44eab3'
+    file_hash = 'e65b0b6bd4b08c2796c3bbea89343748195e29e240ce28e70489c53d06ca69d9'
     model_path = fetch_file('http://data.brainchip.com/models/yolo/' + model_name,
                             fname=model_name,
                             file_hash=file_hash,
