@@ -23,6 +23,8 @@ import tensorflow as tf
 
 from keras import backend as K
 
+from .box_utils import compute_overlap
+
 EPSILON = 1e-7
 
 
@@ -62,7 +64,6 @@ class YoloLoss():
 
         self.__name__ = 'yolo_loss'
         self.iou_filter = iou_filter
-        self.readjust_obj_score = False
 
         self.lambda_coord = lambda_coord
         self.lambda_noobj = lambda_noobj
@@ -137,9 +138,7 @@ class YoloLoss():
             float: the computed loss
         """
 
-        b_o = self._calculate_ious(y_true,
-                                   y_pred,
-                                   use_iou=self.readjust_obj_score)
+        b_o = y_true[..., 4]
         b_o_pred = y_pred[..., 4]
 
         num_true_labels = self.grid_size[0] * \
@@ -147,8 +146,7 @@ class YoloLoss():
         y_true_p = K.reshape(y_true[..., :4],
                              shape=(self.batch_size, 1, 1, 1, num_true_labels,
                                     4))
-        iou_scores_buff = self._calculate_ious(y_true_p,
-                                               K.expand_dims(y_pred, axis=4))
+        iou_scores_buff = compute_overlap(y_true_p, K.expand_dims(y_pred, axis=4))
         best_ious = K.max(iou_scores_buff, axis=4)
 
         indicator_noobj = K.cast(best_ious < self.iou_filter, np.float32) * (
@@ -191,45 +189,3 @@ class YoloLoss():
         loss = total_coord_loss + total_obj_loss + total_class_loss
 
         return loss
-
-    @staticmethod
-    def _calculate_ious(a1, a2, use_iou=True):
-
-        if not use_iou:
-            return a1[..., 4]
-
-        def process_boxes(a):
-            # Align x-w, y-h
-            a_xy = a[..., 0:2]
-            a_wh = a[..., 2:4]
-
-            a_wh_half = a_wh / 2.
-            # Get x_min, y_min
-            a_mins = a_xy - a_wh_half
-            # Get x_max, y_max
-            a_maxes = a_xy + a_wh_half
-
-            return a_mins, a_maxes, a_wh
-
-        # Process two sets
-        a2_mins, a2_maxes, a2_wh = process_boxes(a2)
-        a1_mins, a1_maxes, a1_wh = process_boxes(a1)
-
-        # Intersection as min(Upper1, Upper2) - max(Lower1, Lower2)
-        intersect_mins = K.maximum(a2_mins, a1_mins)
-        intersect_maxes = K.minimum(a2_maxes, a1_maxes)
-
-        # Getting the intersections in the xy (aka the width, height
-        # intersection)
-        intersect_wh = K.maximum(intersect_maxes - intersect_mins, 0.)
-
-        # Multiply to get intersecting area
-        intersect_areas = intersect_wh[..., 0] * intersect_wh[..., 1]
-
-        # Values for the single sets
-        true_areas = a1_wh[..., 0] * a1_wh[..., 1]
-        pred_areas = a2_wh[..., 0] * a2_wh[..., 1]
-
-        # Compute union for the IoU
-        union_areas = pred_areas + true_areas - intersect_areas
-        return intersect_areas / union_areas
