@@ -30,7 +30,7 @@ from cnn2snn import load_quantized_model, convert
 
 from .yolo_loss import YoloLoss
 from .map_evaluation import MapEvaluation
-from .batch_generator import BatchGenerator
+from .batch_generator import BatchYoloGenerator
 from ..training import get_training_parser, freeze_model_before, compile_model
 
 
@@ -74,20 +74,20 @@ def train(model, train_data, valid_data, anchors, labels, obj_threshold,
     # Build batch generators
     input_shape = model.input.shape[1:]
 
-    train_generator = BatchGenerator(input_shape=input_shape,
-                                     data=train_data,
-                                     grid_size=grid_size,
-                                     labels=labels,
-                                     anchors=anchors,
-                                     batch_size=batch_size)
+    train_generator = BatchYoloGenerator(input_shape=input_shape,
+                                         data=train_data,
+                                         grid_size=grid_size,
+                                         labels=labels,
+                                         anchors=anchors,
+                                         batch_size=batch_size)
 
-    valid_generator = BatchGenerator(input_shape=input_shape,
-                                     data=valid_data,
-                                     grid_size=grid_size,
-                                     labels=labels,
-                                     anchors=anchors,
-                                     batch_size=batch_size,
-                                     jitter=False)
+    valid_generator = BatchYoloGenerator(input_shape=input_shape,
+                                         data=valid_data,
+                                         grid_size=grid_size,
+                                         labels=labels,
+                                         anchors=anchors,
+                                         batch_size=batch_size,
+                                         jitter=False)
 
     # Create callbacks
     early_stop_cb = EarlyStopping(monitor='val_loss',
@@ -155,12 +155,12 @@ def extract_samples(im_size, data, grid_size, anchors, labels, num_samples, out_
         num_samples (int): number of samples to extract
         out_file (str): name of output file
     """
-    data_generator = BatchGenerator(input_shape=im_size,
-                                    data=data,
-                                    grid_size=grid_size,
-                                    labels=labels,
-                                    anchors=anchors,
-                                    batch_size=num_samples)
+    data_generator = BatchYoloGenerator(input_shape=im_size,
+                                        data=data,
+                                        grid_size=grid_size,
+                                        labels=labels,
+                                        anchors=anchors,
+                                        batch_size=num_samples)
     np.savez(out_file, data_generator[0][0])
     print(f"Samples saved as {out_file}")
 
