@@ -28,8 +28,8 @@ from keras.utils.np_utils import to_categorical
 from cnn2snn import load_quantized_model, convert
 
 from ..cyclic_lr import CyclicLR
-from ..training import (get_training_parser, compile_model, evaluate_model,
-                        evaluate_akida_model, print_history_stats)
+from ..training import (get_training_parser, compile_model, evaluate_model, evaluate_akida_model,
+                        print_history_stats, RestoreBest)
 from ..utils import fetch_file
 
 
@@ -98,14 +98,22 @@ def train_model(model, train_ds, test_ds, batch_size, max_learning_rate, epochs,
     """
     n_iterations = np.round(num_samples / batch_size)
 
+    # Define training callbacks
+    callbacks = []
+
     lr_scheduler = CyclicLR(base_lr=max_learning_rate * 0.01,
                             max_lr=max_learning_rate,
                             step_size=4 * n_iterations,
                             mode='triangular2')
+    callbacks.append(lr_scheduler)
+
+    # Model checkpoints (save best model and retrieve it when training is complete)
+    restore_model = RestoreBest(model)
+    callbacks.append(restore_model)
 
     history = model.fit(train_ds,
                         epochs=epochs,
-                        callbacks=[lr_scheduler],
+                        callbacks=callbacks,
                         validation_data=test_ds)
     print_history_stats(history)
 
