@@ -20,7 +20,6 @@ Training script for ImageNet models.
 """
 
 import os
-import time
 import argparse
 
 import numpy as np
@@ -31,7 +30,7 @@ except ImportError:
     tfds = None
 
 import keras
-from keras.callbacks import TensorBoard, ModelCheckpoint, LearningRateScheduler
+from keras.callbacks import LearningRateScheduler
 from keras import Sequential
 from keras.optimizers import SGD, Adam
 from tensorflow_addons.optimizers import LAMB
@@ -39,35 +38,17 @@ from keras.layers import Input, Activation
 from keras.models import clone_model
 
 import akida
-from cnn2snn import load_quantized_model, convert
+from cnn2snn import convert
 from quantizeml.layers import AddPositionEmbs, ClassToken
-from quantizeml.models import load_model
 
 from .preprocessing import preprocess_image, DATA_AUGMENTATION
 from ..training import (get_training_parser, calibrate_model, freeze_model_before,
-                        print_history_stats)
+                        print_history_stats, RestoreBest)
 from ..calibrable import calibration_required
 from ..transformers.model_vit import apply_embedding_weights
 from ..distiller import DeitDistiller
-
-
-def _load_model(model_path):
-    """Combine the cnn2snn.load_quantized_model and quantizeml.load_model
-
-    Args:
-        model_path (str): model path
-
-    Returns:
-        keras.Model: the load model
-    """
-    try:
-        model = load_quantized_model(model_path)
-    except Exception:
-        try:
-            model = load_model(model_path)
-        except Exception as e:
-            raise e.__class__('Cannot load provided model.')
-    return model
+from ..param_scheduler import get_cosine_lr_scheduler
+from ..utils import get_tensorboard_callback, load_model
 
 
 def get_imagenet_dataset(data_path, training, image_size, batch_size, data_aug=True, one_hot=False):
@@ -293,7 +274,7 @@ def rescale_vit(base_model, input_size):
     for base_layer, clone_layer in zip(base_model.layers, clone.layers):
         base_weights = base_layer.get_weights()
         if isinstance(base_layer, AddPositionEmbs):
-            apply_embedding_weights(clone_layer, base_weights, x_patch, y_patch, num_tokens)
+            apply_embedding_weights(clone_layer, base_weights, (x_patch, y_patch), num_tokens)
         else:
             clone_layer.set_weights(base_weights)
     return clone
@@ -315,28 +296,6 @@ def rescale(base_model, input_size):
     return rescale_legacy(base_model, input_size)
 
 
-def create_log_dir(out_dir):
-    """ Creates a directory to store Tensorflow logs.
-
-    Args:
-        out_dir (str): parent directory of the folder to create
-
-    Returns:
-        str: full path of the created directory
-    """
-    base_name = 'imagenet_cnn' + '_' + time.strftime('%Y_%m_%d.%H_%M_%S',
-                                                     time.localtime())
-    log_dir = os.path.join(out_dir, base_name)
-
-    print('saving tensorboard and checkpoint information to', log_dir)
-    if not os.path.exists(log_dir):
-        os.makedirs(log_dir)
-        print('directory', log_dir, 'created ...')
-    else:
-        print('directory', log_dir, 'already exists ...')
-    return log_dir
-
-
 def train(model,
           train_dataset,
           train_steps,
@@ -347,7 +306,7 @@ def train(model,
           tune=False,
           learning_rate=1e-1,
           initial_epoch=0,
-          lrs_cos=False):
+          lr_policy='exp_decay'):
     """ Trains the model
 
     Args:
@@ -363,8 +322,9 @@ def train(model,
         learning_rate (float, optional): the learning rate. Defaults to 1e-1.
         initial_epoch (int, optional): epoch at which to start training.
           Defaults to 0.
-        lrs_cos (bool, optional): True to use cosine learning rate scheduler instead of the default
-          exponential rule. Defaults to False.
+        lr_policy (str, optional): defines the learning rate policy to adopt. Values in
+            ['exp_decay', 'cosine_decay', 'cosine_sched'] for exponential decay, cosine decay and
+            cosine oscillation respectively. Defaults to 'exp_decay'.
     """
     # 1. Check model validity
     if calibration_required(model):
@@ -374,8 +334,32 @@ def train(model,
     # 2. Define training callbacks
     callbacks = []
 
-    if lrs_cos:
-        # 2.1 Learning rate scheduler, modified by tune procedure
+    # 2.1 Learning rate scheduler
+    if lr_policy == 'exp_decay':
+        LR_START = learning_rate
+        LR_END = 1e-4
+        # number of epochs you first keep the learning rate constant
+        LR_EPOCH_CONSTANT = 10
+        # Modify default values for fine-tuning
+        if tune:
+            LR_START = 1e-4
+            LR_END = 1e-8
+            LR_EPOCH_CONSTANT = 2
+
+        if LR_EPOCH_CONSTANT >= num_epochs:
+            lr_decay = LR_END / LR_START
+        else:
+            lr_decay = (LR_END / LR_START)**(1. / (num_epochs - LR_EPOCH_CONSTANT))
+
+        # This function keeps the learning rate at LR_START for the first N epochs
+        # and decreases it exponentially after that.
+        def agg_lr_scheduler(epoch):
+            if epoch < LR_EPOCH_CONSTANT:
+                return LR_START
+            return LR_START * lr_decay**(epoch - (LR_EPOCH_CONSTANT - 1))
+
+        lr_scheduler = LearningRateScheduler(agg_lr_scheduler)
+    elif lr_policy == 'cosine_decay':
         LR_START = learning_rate
         if tune:
             LR_EPOCH_CONSTANT = 2  # number of epochs you first keep the learning rate constant
@@ -400,52 +384,19 @@ def train(model,
             cosine_decay = 0.5 * (1 + np.cos(np.pi * step / (num_epochs - LR_EPOCH_CONSTANT)))
             decayed = (1 - LR_MIN) * cosine_decay + LR_MIN
             return LR_START * decayed
-
-        callbacks.append(LearningRateScheduler(cos_lr_scheduler))
+        lr_scheduler = LearningRateScheduler(cos_lr_scheduler)
+    elif lr_policy == 'cosine_sched':
+        lr_scheduler = get_cosine_lr_scheduler(learning_rate, num_epochs * train_steps)
     else:
-        # 2.1 Learning rate scheduler
-        LR_START = learning_rate
-        LR_END = 1e-4
-        # number of epochs you first keep the learning rate constant
-        LR_EPOCH_CONSTANT = 10
-        # Modify default values for fine-tuning
-        if tune:
-            LR_START = 1e-4
-            LR_END = 1e-8
-            LR_EPOCH_CONSTANT = 2
+        raise ValueError(f"Unsupported learning rate policy '{lr_policy}'.")
+    callbacks.append(lr_scheduler)
 
-        if LR_EPOCH_CONSTANT >= num_epochs:
-            lr_decay = LR_END / LR_START
-        else:
-            lr_decay = (LR_END / LR_START)**(1. / (num_epochs - LR_EPOCH_CONSTANT))
-
-        # This function keeps the learning rate at LR_START for the first N epochs
-        # and decreases it exponentially after that.
-        def agg_lr_scheduler(epoch):
-            if epoch < LR_EPOCH_CONSTANT:
-                return LR_START
-            return LR_START * lr_decay**(epoch - (LR_EPOCH_CONSTANT - 1))
-
-        callbacks.append(LearningRateScheduler(agg_lr_scheduler))
-
-    # 2.2 Model checkpoints (save best models after each epoch)
-    log_dir = create_log_dir(out_dir)
-    filepath = os.path.join(log_dir, 'best_weights.h5')
-    model_checkpoint = ModelCheckpoint(filepath=filepath,
-                                       monitor='val_accuracy',
-                                       verbose=1,
-                                       save_best_only=True,
-                                       mode='max')
-    callbacks.append(model_checkpoint)
+    # 2.2 Model checkpoints (save best model and retrieve it when training is complete)
+    restore_model = RestoreBest(model)
+    callbacks.append(restore_model)
 
     # 2.3 Tensorboard logs
-    file_writer = tf.summary.create_file_writer(log_dir + "/metrics")
-    file_writer.set_as_default()
-    tensorboard = TensorBoard(log_dir=log_dir,
-                              histogram_freq=1,
-                              update_freq='epoch',
-                              write_graph=False,
-                              profile_batch=0)
+    tensorboard = get_tensorboard_callback(out_dir, prefix='imagenet')
     callbacks.append(tensorboard)
 
     # 3. Train
@@ -458,10 +409,6 @@ def train(model,
                         initial_epoch=initial_epoch)
     print_history_stats(history)
 
-    # 4. Restore best weights
-    if os.path.exists(filepath):
-        model.load_weights(filepath)
-
 
 def main():
     """ Entry point for script and CLI usage.
@@ -499,10 +446,13 @@ def main():
     train_parser.add_argument("--optim", type=str, default="SGD",
                               help="Optimizer to use. Defaults to %(default)s.")
     train_parser.add_argument("--data_aug", action='store_true', help="Enables custom DA.")
-    train_parser.add_argument("--lrs_cos",
-                              action='store_true',
-                              help="Use cosine learning rate scheduler instead of the exponential \
-                                    default.")
+    train_parser.add_argument("--lr_policy",
+                              default='exp_decay',
+                              choices=['exp_decay', 'cosine_decay', 'cosine_sched'],
+                              help="Defines the learning rate scheduling. Values in "
+                                   "['exp_decay', 'cosine_decay', 'cosine_sched'] which corresponds"
+                                   " to exponential decay, cosine decay and cosine oscillation "
+                                   "respectively.")
     train_parser.add_argument("--teacher", type=str, default=None,
                               help="Teacher model use to train the model through Knowledge"
                               "Distillation. The input model output an Add layer. "
@@ -530,10 +480,13 @@ def main():
                              type=str,
                              default="SGD",
                              help="Optimizer to use. Defaults to %(default)s.")
-    tune_parser.add_argument("--lrs_cos",
-                             action='store_true',
-                             help="Use cosine learning rate scheduler instead of the exponential \
-                                   default.")
+    tune_parser.add_argument("--lr_policy",
+                             default='exp_decay',
+                             choices=['exp_decay', 'cosine_decay', 'cosine_sched'],
+                             help="Defines the learning rate scheduling. Values in "
+                                  "['exp_decay', 'cosine_decay', 'cosine_sched'] which corresponds "
+                                  "to exponential decay, cosine decay and cosine oscillation "
+                                  "respectively.")
     tune_parser.add_argument("--teacher", type=str, default=None,
                              help="Teacher model use to train the model through Knowledge"
                              "Distillation. The input model output an Add layer. "
@@ -568,14 +521,14 @@ def main():
     args = parsers[0].parse_args()
 
     # Load the source model
-    model = _load_model(args.model)
+    model = load_model(args.model)
 
     # Try to load the teacher model, used after to train with Knowledge Distillation
     # Hyperparameters takes from https://arxiv.org/pdf/2012.12877.pdf
     if getattr(args, 'teacher', False):
         if args.distiller_type not in ['none', 'soft', 'hard']:
             raise ValueError("Distiller type must be one of ['none', 'soft', 'hard']")
-        teacher = _load_model(args.teacher)
+        teacher = load_model(args.teacher)
         train_model = DeitDistiller(model, teacher, alpha=args.alpha, temperature=3.0)
     else:
         train_model = model
@@ -606,7 +559,7 @@ def main():
         tune = args.action == 'tune'
 
         learning_rate = args.learning_rate
-        if args.lrs_cos:
+        if args.lr_policy == 'cosine_decay':
             # Tune learning rate following https://arxiv.org/pdf/2012.12877.pdf
             learning_rate *= args.batch_size / 512
 
@@ -620,7 +573,7 @@ def main():
               tune=tune,
               learning_rate=learning_rate,
               initial_epoch=args.initial_epoch,
-              lrs_cos=args.lrs_cos)
+              lr_policy=args.lr_policy)
 
         # Save model in Keras format (h5)
         if args.savemodel:
