@@ -27,68 +27,18 @@ compatible for Akida with:
     - different initialization options.
 """
 
-import warnings
-
 from keras import Model, regularizers
 from keras.layers import (Input, Reshape, Dropout, Activation, Rescaling)
 
 from cnn2snn import load_quantized_model, quantize
 
+from .imagenet_utils import obtain_input_shape
 from ..layer_blocks import conv_block, separable_conv_block
 from ..utils import fetch_file
 
 BASE_WEIGHT_PATH = 'http://data.brainchip.com/models/mobilenet/'
 
 
-def _obtain_input_shape(input_shape, default_size, min_size, include_top):
-    """Internal utility to compute/validate a model's input shape.
-
-    Args:
-        input_shape: either None (will return the default model input shape),
-            or a user-provided shape to be validated.
-        default_size: default input width/height for the model.
-        min_size: minimum input width/height accepted by the model.
-        include_top: whether the model is expected to
-            be linked to a classifier via a Flatten layer.
-
-    Returns:
-        tuple of integers: input shape (may include None entries).
-
-    Raises:
-        ValueError: in case of invalid argument values.
-    """
-    if input_shape and len(input_shape) == 3:
-        if input_shape[-1] not in {1, 3}:
-            warnings.warn('This model usually expects 1 or 3 input channels. '
-                          'However, it was passed an input_shape with ' +
-                          str(input_shape[-1]) + ' input channels.')
-        default_shape = (default_size, default_size, input_shape[-1])
-    else:
-        default_shape = (default_size, default_size, 3)
-
-    if input_shape:
-        if input_shape is not None:
-            if len(input_shape) != 3:
-                raise ValueError(
-                    '`input_shape` must be a tuple of three integers.')
-            if ((input_shape[0] is not None and input_shape[0] < min_size) or
-                    (input_shape[1] is not None and input_shape[1] < min_size)):
-                raise ValueError('Input size must be at least ' +
-                                 str(min_size) + 'x' + str(min_size) +
-                                 '; got `input_shape=' + str(input_shape) + '`')
-    else:
-        if include_top:
-            input_shape = default_shape
-        else:
-            input_shape = (None, None, 3)
-    if include_top:
-        if None in input_shape:
-            raise ValueError('If `include_top` is True, '
-                             'you should specify a static `input_shape`. '
-                             'Got `input_shape=' + str(input_shape) + '`')
-    return input_shape
-
-
 def mobilenet_imagenet(input_shape=None,
                        alpha=1.0,
                        dropout=1e-3,
@@ -103,8 +53,9 @@ def mobilenet_imagenet(input_shape=None,
     """Instantiates the MobileNet architecture.
 
     Args:
-        input_shape (tuple): optional shape tuple.
-        alpha (float): controls the width of the model.
+        input_shape (tuple, optional): shape tuple. Defaults to None.
+        alpha (float, optional): controls the width of the model.
+            Defaults to 1.0.
 
             * If `alpha` < 1.0, proportionally decreases the number of filters
               in each layer.
@@ -112,33 +63,36 @@ def mobilenet_imagenet(input_shape=None,
               in each layer.
             * If `alpha` = 1, default number of filters from the paper are used
               at each layer.
-        dropout (float): dropout rate
-        include_top (bool): whether to include the fully-connected
-            layer at the top of the model.
-        pooling (str): Optional pooling mode for feature extraction
+        dropout (float, optional): dropout rate. Defaults to 1e-3.
+        include_top (bool, optional): whether to include the fully-connected
+            layer at the top of the model. Defaults to True.
+        pooling (str, optional): optional pooling mode for feature extraction
             when `include_top` is `False`.
+            Defaults to None.
 
             * `None` means that the output of the model will be the 4D tensor
               output of the last convolutional block.
             * `avg` means that global average pooling will be applied to the
               output of the last convolutional block, and thus the output of the
               model will be a 2D tensor.
-        classes (int): optional number of classes to classify images
-            into, only to be specified if `include_top` is True.
-        use_stride2 (bool): optional, replace max pooling operations by stride 2
-            convolutions in layers separable 2, 4, 6 and 12.
-        weight_quantization (int): sets all weights in the model to have
+        classes (int, optional): optional number of classes to classify images
+            into, only to be specified if `include_top` is `True`. Defaults to 1000.
+        use_stride2 (bool, optional): replace max pooling operations by stride 2
+            convolutions in layers separable 2, 4, 6 and 12. Defaults to False.
+        weight_quantization (int, optional): sets all weights in the model to have
             a particular quantization bitwidth except for the weights in the
             first layer.
+            Defaults to 0.
 
             * '0' implements floating point 32-bit weights.
             * '2' through '8' implements n-bit weights where n is from 2-8 bits.
-        activ_quantization: sets all activations in the model to have a
+        activ_quantization (int, optional): sets all activations in the model to have a
             particular activation quantization bitwidth.
+            Defaults to 0.
 
             * '0' implements floating point 32-bit activations.
             * '2' through '8' implements n-bit weights where n is from 2-8 bits.
-        input_weight_quantization: sets weight quantization in the first layer.
+        input_weight_quantization (int, optional): sets weight quantization in the first layer.
             Defaults to weight_quantization value.
 
             * '0' implements floating point 32-bit weights.
@@ -181,10 +135,10 @@ def mobilenet_imagenet(input_shape=None,
         else:
             default_size = 224
 
-    input_shape = _obtain_input_shape(input_shape,
-                                      default_size=default_size,
-                                      min_size=32,
-                                      include_top=include_top)
+    input_shape = obtain_input_shape(input_shape,
+                                     default_size=default_size,
+                                     min_size=32,
+                                     include_top=include_top)
 
     rows = input_shape[0]
     cols = input_shape[1]
