@@ -27,7 +27,7 @@ from cnn2snn import load_quantized_model, quantize_layer, convert
 
 from ..cyclic_lr import CyclicLR
 from ..training import (get_training_parser, calibrate_model, compile_model, evaluate_model,
-                        print_history_stats)
+                        print_history_stats, RestoreBest)
 from ..utils import fetch_file
 
 
@@ -80,6 +80,10 @@ def train_model(model, x_train, y_train, x_val, y_val, batch_size, epochs):
                    mode='triangular')
     callbacks.append(clr)
 
+    # Model checkpoints (save best model and retrieve it when training is complete)
+    restore_model = RestoreBest(model)
+    callbacks.append(restore_model)
+
     history = model.fit(x_train,
                         to_categorical(y_train),
                         batch_size=batch_size,
