@@ -21,12 +21,13 @@ from functools import partial
 
 from keras.layers import (BatchNormalization, ReLU, Conv2D, SeparableConv2D, Dense, MaxPool2D,
                           AvgPool2D, GlobalAvgPool2D, LayerNormalization, Dropout, Add,
-                          Conv2DTranspose)
+                          Conv2DTranspose, Reshape)
 from keras.activations import swish
 from keras.initializers import TruncatedNormal
 from tensorflow_addons.layers import GELU, GroupNormalization
 
-from quantizeml.layers import Attention, LayerMadNormalization, SeparableConv2DTranspose
+from quantizeml.layers import (Attention, LayerMadNormalization, SeparableConv2DTranspose,
+                               ExtractToken)
 
 
 def _add_pooling_layer(x, pooling_type, pool_size, padding, layer_base_name):
@@ -458,9 +459,59 @@ def sepconv_transpose_block(inputs,
     x = sepconv_trans_layer(inputs)
 
     if add_batchnorm:
-        x = BatchNormalization(name=sepconv_trans_layer.name + '_BN')(x)
+        x = BatchNormalization(name=sepconv_trans_layer.name + '/BN')(x)
 
     if add_activation:
-        x = ReLU(8.0, name=sepconv_trans_layer.name + '_relu')(x)
+        x = ReLU(8.0, name=sepconv_trans_layer.name + '/relu')(x)
 
     return x
+
+
+def yolo_head_block(x, num_boxes, classes, filters=1024):
+    """Adds the `YOLOv2 detection head <https://arxiv.org/pdf/1612.08242.pdf>`_, at the output
+    of a model.
+
+    Args:
+        x (:obj:`tf.Tensor`): input tensor of shape `(rows, cols, channels)`.
+        num_boxes (int): number of boxes.
+        classes (int): number of classes.
+        filters (int, optional): number of filters in hidden layers. Defaults to 1024.
+
+    Returns:
+        :obj:`tf.Tensor`: output tensor of yolo detection head block.
+
+    Notes:
+        This block replaces conv layers by separable_conv, to decrease the amount of parameters.
+    """
+    x = separable_conv_block(x, filters=filters, name='1conv',
+                             kernel_size=(3, 3), padding='same', use_bias=False,
+                             add_activation=True, add_batchnorm=True)
+    x = separable_conv_block(x, filters=filters, name='2conv',
+                             kernel_size=(3, 3), padding='same', use_bias=False,
+                             add_activation=True, add_batchnorm=True)
+    x = separable_conv_block(x, filters=filters, name='3conv',
+                             kernel_size=(3, 3), padding='same', use_bias=False,
+                             add_activation=True, add_batchnorm=True)
+    x = separable_conv_block(x, filters=(num_boxes * (4 + 1 + classes)), name='detection_layer',
+                             kernel_size=(3, 3), padding='same', use_bias=True,
+                             add_activation=False, add_batchnorm=False)
+    return x
+
+
+def vit_extract_feature_map(y, feat_shape, num_non_patch=1):
+    """Add a ExtractToken + Reshape layers to convert the plain ViT output into a feature map.
+
+    All tokens will be extracted except those considered as class tokens (``num_non_patch``)
+
+    Args:
+        y (:obj:`tf.Tensor`): output of ViT classifier model.
+        feat_shape (tuple of int): height/width desired output size
+        num_non_patch (int, optional): number of non-patch tokens to exclude on the final output.
+            Defaults to 1.
+
+    Returns:
+        :obj:`keras.Model`: ViT with new layers
+    """
+    y = ExtractToken(name="extract_features", token=list(range(num_non_patch, y.shape[1])))(y)
+    y = Reshape(target_shape=(*feat_shape, y.shape[-1]), name="features")(y)
+    return y
