@@ -27,9 +27,10 @@ from keras.metrics import BinaryIoU
 
 from cnn2snn import load_quantized_model
 
-from akida_models.param_scheduler import ParamScheduler, combine_scheds, SchedCos
+from akida_models.param_scheduler import get_cosine_lr_scheduler
 
-from ..training import get_training_parser, compile_model, evaluate_model, print_history_stats
+from ..training import (get_training_parser, compile_model, evaluate_model, print_history_stats,
+                        RestoreBest)
 
 
 def get_data(path, batch_size):
@@ -102,23 +103,11 @@ def train_model(model, train_gen, steps_per_epoch, val_gen, val_steps, epochs, l
         learning_rate (float): the learning rate
     """
     # Define learning rate scheduler
-    mom_max = 0.95
-    mom_min = 0.85
+    callbacks = [get_cosine_lr_scheduler(learning_rate, epochs * steps_per_epoch, True)]
 
-    div = 25.
-    final_div = 25e4
-    pct_start = 0.3
-
-    # Define a scheduling policy for both the learning rate and the Adam decay where two phases are
-    # defined (30% of the time and the other 70%) and during which values will follow a cosine
-    # oscillation within specific boundaries.
-    scheds = {'learning_rate': combine_scheds([pct_start, 1 - pct_start],
-                                              [SchedCos(learning_rate / div, learning_rate),
-                                               SchedCos(learning_rate, learning_rate / final_div)]),
-              "beta_1": combine_scheds([pct_start, 1 - pct_start],
-                                       [SchedCos(mom_max, mom_min),
-                                        SchedCos(mom_min, mom_max)])}
-    callbacks = [ParamScheduler(scheds, total_num_iterations=epochs * steps_per_epoch)]
+    # Model checkpoints (save best model and retrieve it when training is complete)
+    restore_model = RestoreBest(model, monitor="val_binary_io_u")
+    callbacks.append(restore_model)
 
     history = model.fit(train_gen,
                         epochs=epochs,
