@@ -32,7 +32,7 @@ BASE_WEIGHT_PATH = 'http://data.brainchip.com/models/deit/'
 
 
 def deit_imagenet(input_shape,
-                  num_layers,
+                  num_blocks,
                   hidden_size,
                   num_heads,
                   name,
@@ -43,13 +43,14 @@ def deit_imagenet(input_shape,
                   include_top=True,
                   distilled=False,
                   norm='LN',
+                  last_norm='LN',
                   softmax='softmax',
                   act="GeLU"):
     """Build a DeiT model.
 
     Args:
         input_shape (tuple): image shape tuple
-        num_layers (int): the number of transformer layers to use.
+        num_blocks (int): the number of transformer blocks to use.
         hidden_size (int): the number of filters to use
         num_heads (int): the number of transformer heads
         name (str): the model name
@@ -59,12 +60,15 @@ def deit_imagenet(input_shape,
         classes (int, optional): number of classes to classify images into, only to be specified if
             `include_top` is True. Defaults to 1000.
         dropout (float, optional): fraction of the units to drop for dense layers. Defaults to 0.1.
-        include_top (bool, optional): Whether to include the final classification layer. If not,
-            the output will have dimensions (batch_size, hidden_size). Defaults to True.
+        include_top (bool, optional): whether to include the final classifier head. If False,
+            the output will correspond to that of the transformer. Defaults to True.
         distilled (bool, optional): Build model append a distilled token. Defaults to False.
         norm (str, optional): string that values in ['LN', 'GN1', 'BN', 'LMN'] and that allows to
             choose from LayerNormalization, GroupNormalization(groups=1, ...), BatchNormalization
             or LayerMadNormalization layers respectively in the model. Defaults to 'LN'.
+        last_norm (str, optional): string that values in ['LN', 'BN']
+            and that allows to choose from LayerNormalization or
+            BatchNormalization in the classifier network. Defaults to 'LN'.
         softmax (str, optional): string with values in ['softmax', 'softmax2']
             that allows to choose between softmax and softmax2 in MHA. Defaults
             to 'softmax'.
@@ -74,10 +78,14 @@ def deit_imagenet(input_shape,
     assert ((input_shape[0] % patch_size == 0) and
             (input_shape[1] % patch_size == 0)), "image size must be a multiple of patch_size"
 
+    if last_norm not in ('LN', 'BN'):
+        raise NotImplementedError("last_norm should be in ['LN', 'BN']"
+                                  f"but received {norm}.")
+
     # Normalize image adding rescaling layer
     x = keras.layers.Input(shape=input_shape, name="input")
-    scale = tuple(1.0 / 255 / std for std in IMAGENET_STD)
-    offset = tuple(-mean / std for mean, std in zip(IMAGENET_MEAN, IMAGENET_STD))
+    scale = list(1.0 / 255 / std for std in IMAGENET_STD)
+    offset = list(-mean / std for mean, std in zip(IMAGENET_MEAN, IMAGENET_STD))
     y = keras.layers.Rescaling(scale=scale, offset=offset, name="Rescale")(x)
 
     # Build model
@@ -95,7 +103,7 @@ def deit_imagenet(input_shape,
         y = ClassToken(name="DistToken")(y)
     y = ClassToken(name="ClassToken")(y)
     y = AddPositionEmbs(name="Transformer/PosEmbed")(y)
-    for n in range(num_layers):
+    for n in range(num_blocks):
         y, _ = transformer_block(
             y,
             num_heads=num_heads,
@@ -107,16 +115,17 @@ def deit_imagenet(input_shape,
             softmax=softmax,
             mlp_act=act,
         )
-    y = norm_to_layer(norm)(epsilon=1e-6, name="Transformer/EncoderNorm")(y)
-    if distilled:
-        yd = ExtractToken(token=1, name="ExtractToken_Dist")(y)
-    y = ExtractToken(token=0, name="ExtractToken")(y)
+
+    # Include classification head
     if include_top:
+        yt = norm_to_layer(last_norm)(
+            epsilon=1e-6, name="Transformer/EncoderNorm")(y)
+        y = ExtractToken(token=0, name="ExtractToken")(yt)
         y = keras.layers.Dense(classes, name="Head")(y)
         if distilled:
+            yd = ExtractToken(token=1, name="ExtractToken_Dist")(yt)
             yd = keras.layers.Dense(classes, name="DistHead")(yd)
-    if distilled:
-        y = Add(name="Add", average=True)([y, yd])
+            y = Add(name="Add", average=True)([y, yd])
 
     # Add distilled flag
     model = keras.models.Model(inputs=x, outputs=y, name=name)
@@ -124,8 +133,14 @@ def deit_imagenet(input_shape,
     return model
 
 
-def deit_ti16(input_shape=(224, 224, 3), classes=1000, distilled=False, norm='LN',
-              softmax='softmax', act='GeLU'):
+def deit_ti16(input_shape=(224, 224, 3),
+              classes=1000,
+              distilled=False,
+              norm='LN',
+              last_norm='LN',
+              softmax='softmax',
+              act='GeLU',
+              include_top=True):
     """Build DeiT-Tiny.
 
     Args:
@@ -135,10 +150,15 @@ def deit_ti16(input_shape=(224, 224, 3), classes=1000, distilled=False, norm='LN
         norm (str, optional): string that values in ['LN', 'GN1', 'BN', 'LMN'] and that allows to
             choose from LayerNormalization, GroupNormalization(groups=1, ...), BatchNormalization
             or LayerMadNormalization layers respectively in the model. Defaults to 'LN'.
+        last_norm (str, optional): string that values in ['LN', 'BN']
+            and that allows to choose from LayerNormalization or
+            BatchNormalization in the classifier network. Defaults to 'LN'.
         softmax (str, optional): string with values in ['softmax', 'softmax2'] that allows to choose
             between softmax and softmax2 in attention block. Defaults to 'softmax'.
         act (str, optional): string that values in ['GeLU', 'ReLUx', 'swish'] and that allows to
             choose from GeLU, ReLUx or swish activation inside MLP. Defaults to 'GeLU'.
+        include_top (bool, optional): whether to include the final classifier network.
+            Defaults to True.
 
     Returns:
         keras.Model: the requested model
@@ -149,51 +169,46 @@ def deit_ti16(input_shape=(224, 224, 3), classes=1000, distilled=False, norm='LN
         classes=classes,
         distilled=distilled,
         norm=norm,
+        last_norm=last_norm,
         act=act,
         softmax=softmax,
+        include_top=include_top,
         **CONFIG_TI,
     )
 
 
-def bc_deit_ti16(input_shape=(224, 224, 3), classes=1000, distilled=False):
+def bc_deit_ti16(input_shape=(224, 224, 3), classes=1000, distilled=False, include_top=True,
+                 num_blocks=12):
     """Build DeiT-Tiny, changing all LN by LMN, using softmax2 and ReLU8.
 
     Args:
         input_shape (tuple, optional): input shape. Defaults to (224, 224, 3).
         classes (int, optional): number of classes. Defaults to 1000.
         distilled (bool, optional): build model appending a distilled token. Defaults to False.
+        include_top (bool, optional): whether to include the final classifier network.
+            Defaults to True.
+        num_blocks (int, optional): the number of transformer blocks to use. Defaults to 12.
 
     Returns:
         keras.Model: the requested model
     """
+    config_ti = CONFIG_TI.copy()
+    config_ti["num_blocks"] = num_blocks
+
     return deit_imagenet(
         name="deit-tiny",
         input_shape=input_shape,
         classes=classes,
         distilled=distilled,
         norm="LMN",
+        last_norm="BN",
         softmax="softmax2",
         act="ReLU8",
-        **CONFIG_TI,
+        include_top=include_top,
+        **config_ti,
     )
 
 
-def bc_deit_ti16_imagenet_pretrained():
-    """ Helper method to retrieve a `bc_deit_ti16` model that was trained on ImageNet dataset.
-
-    Returns:
-        keras.Model, dict: a Keras Model instance and the quantization configuration as a JSON
-        formatted dict.
-    """
-    model_name = 'bc_deit_ti16_224_quant_config_4bit.h5'
-    file_hash = 'b702c791170cfe63dcf50e2d873144ea2d8c2ecb2d9de3b59884fc28034a38d0'
-    model_path = fetch_file(BASE_WEIGHT_PATH + model_name,
-                            fname=model_name,
-                            file_hash=file_hash,
-                            cache_subdir='models')
-    return load_model(model_path)
-
-
 def bc_deit_dist_ti16_imagenet_pretrained():
     """ Helper method to retrieve a `bc_deit_dist_ti16` model that was trained on ImageNet dataset.
 
@@ -202,7 +217,7 @@ def bc_deit_dist_ti16_imagenet_pretrained():
         formatted dict.
     """
     model_name = 'bc_deit_dist_ti16_224_quant_config_4bit.h5'
-    file_hash = 'affd7e084486fd5329567712326a5038c4aae25bb778284959c0179fbf5238ce'
+    file_hash = '0d8b1c46e215be149974e6116e2e88a50a9c6d2186f7131f4fb0b3f23aaa2aac'
     model_path = fetch_file(BASE_WEIGHT_PATH + model_name,
                             fname=model_name,
                             file_hash=file_hash,
@@ -210,13 +225,15 @@ def bc_deit_dist_ti16_imagenet_pretrained():
     return load_model(model_path)
 
 
-def deit_s16(input_shape=(224, 224, 3), classes=1000, distilled=False):
+def deit_s16(input_shape=(224, 224, 3), classes=1000, distilled=False, include_top=True):
     """Build DeiT-Small.
 
     Args:
         input_shape (tuple, optional): input shape. Defaults to (224, 224, 3).
         classes (int, optional): number of classes. Defaults to 1000.
         distilled (bool, optional): build model appending a distilled token. Defaults to False.
+        include_top (bool, optional): whether to include the final classifier network.
+            Defaults to True.
 
     Returns:
         keras.Model: the requested model
@@ -226,17 +243,20 @@ def deit_s16(input_shape=(224, 224, 3), classes=1000, distilled=False):
         input_shape=input_shape,
         classes=classes,
         distilled=distilled,
+        include_top=include_top,
         **CONFIG_S,
     )
 
 
-def deit_b16(input_shape=(224, 224, 3), classes=1000, distilled=False):
+def deit_b16(input_shape=(224, 224, 3), classes=1000, distilled=False, include_top=True):
     """Build DeiT-B16.
 
     Args:
         input_shape (tuple, optional): input shape. Defaults to (224, 224, 3).
         classes (int, optional): number of classes. Defaults to 1000.
         distilled (bool, optional): build model appending a distilled token. Defaults to False.
+        include_top (bool, optional): whether to include the final classifier network.
+            Defaults to True.
 
     Returns:
         keras.Model: the requested model
@@ -246,5 +266,6 @@ def deit_b16(input_shape=(224, 224, 3), classes=1000, distilled=False):
         input_shape=input_shape,
         classes=classes,
         distilled=distilled,
+        include_top=include_top,
         **CONFIG_B,
     )
