@@ -40,7 +40,7 @@ ConfigDict = tx.TypedDict(
         "dropout": float,
         "mlp_dim": int,
         "num_heads": int,
-        "num_layers": int,
+        "num_blocks": int,
         "hidden_size": int,
     },
 )
@@ -49,7 +49,7 @@ CONFIG_TI: ConfigDict = {
     "dropout": 0.0,
     "mlp_dim": 768,
     "num_heads": 3,
-    "num_layers": 12,
+    "num_blocks": 12,
     "hidden_size": 192,
 }
 
@@ -57,7 +57,7 @@ CONFIG_S: ConfigDict = {
     "dropout": 0.1,
     "mlp_dim": 1536,
     "num_heads": 6,
-    "num_layers": 12,
+    "num_blocks": 12,
     "hidden_size": 384,
 }
 
@@ -65,7 +65,7 @@ CONFIG_B: ConfigDict = {
     "dropout": 0.1,
     "mlp_dim": 3072,
     "num_heads": 12,
-    "num_layers": 12,
+    "num_blocks": 12,
     "hidden_size": 768,
 }
 
@@ -73,29 +73,35 @@ CONFIG_L: ConfigDict = {
     "dropout": 0.1,
     "mlp_dim": 4096,
     "num_heads": 16,
-    "num_layers": 24,
+    "num_blocks": 24,
     "hidden_size": 1024,
 }
 
 
-def apply_embedding_weights(target_layer, source_weights, num_x_patches, num_y_patches,
-                            num_tokens=1):
+def apply_embedding_weights(target_layer, source_weights, num_patches, num_tokens=1):
     """Apply embedding weights to a target layer.
 
     Args:
         target_layer (:obj:`keras.Layer`): The target layer to which weights will be applied.
         source_weights (list of :obj:`np.array`): The source weights.
-        num_x_patches (int, optional): Number of x-patches in embedding layer.
-        num_y_patches (int, optional): Number of y-patches in embedding layer.
+        num_patches (int or tuple): Number of patches in xy spatial dimension, extracted from
+            embedding layer.
         num_tokens (int, optional): Number of tokens. Defaults to 1.
     """
     assert isinstance(source_weights, list), "source_weights must be a list of numpy arrays"
     expected_shape = target_layer.weights[0].shape
+
+    # number of patches constrains
+    if isinstance(num_patches, int):
+        num_patches = (num_patches, num_patches)
+    assert len(num_patches) == 2, "num_patches must contain at most two values"
+    assert all(x > 0 for x in num_patches), "every patch must be greater than 0"
+
     if expected_shape != source_weights[0].shape:
         token, grid = source_weights[0][0, :num_tokens], source_weights[0][0, num_tokens:]
         sin = int(np.sqrt(grid.shape[0]))
-        sout_x = num_x_patches
-        sout_y = num_y_patches
+        sout_x = num_patches[0]
+        sout_y = num_patches[1]
         warnings.warn(
             "Resizing position embeddings from " f"{sin}, {sin} to {sout_x}, {sout_y}",
             UserWarning,
@@ -111,7 +117,7 @@ def apply_embedding_weights(target_layer, source_weights, num_x_patches, num_y_p
 
 def vit_imagenet(input_shape,
                  patch_size,
-                 num_layers,
+                 num_blocks,
                  hidden_size,
                  num_heads,
                  name,
@@ -120,6 +126,7 @@ def vit_imagenet(input_shape,
                  dropout=0.1,
                  include_top=True,
                  norm='LN',
+                 last_norm='LN',
                  softmax='softmax',
                  act="GeLU"):
     """Build a ViT model.
@@ -127,7 +134,7 @@ def vit_imagenet(input_shape,
     Args:
         input_shape (tuple): image shape tuple
         patch_size (int): the size of each patch (must fit evenly in image size)
-        num_layers (int): the number of transformer layers to use.
+        num_blocks (int): the number of transformer blocks to use.
         hidden_size (int): the number of filters to use
         num_heads (int): the number of transformer heads
         name (str): the model name
@@ -135,11 +142,14 @@ def vit_imagenet(input_shape,
         classes (int, optional): number of classes to classify images into, only to be specified if
             `include_top` is True. Defaults to 1000.
         dropout (float, optional): fraction of the units to drop for dense layers. Defaults to 0.1.
-        include_top (bool, optional): whether to include the final classification layer. If not,
-            the output will have dimensions (batch_size, hidden_size). Defaults to True.
+        include_top (bool, optional): whether to include the final classifier head. If False,
+            the output will correspond to that of the transformer. Defaults to True.
         norm (str, optional): string that values in ['LN', 'GN1', 'BN', 'LMN'] and that allows to
             choose from LayerNormalization, GroupNormalization(groups=1, ...), BatchNormalization
             or LayerMadNormalization layers respectively in the model. Defaults to 'LN'.
+        last_norm (str, optional): string that values in ['LN', 'BN']
+            and that allows to choose from LayerNormalization or
+            BatchNormalization in the classifier network. Defaults to 'LN'.
         softmax (str, optional): string with values in ['softmax', 'softmax2']
             that allows to choose between softmax and softmax2 in MHA. Defaults
             to 'softmax'.
@@ -149,6 +159,10 @@ def vit_imagenet(input_shape,
     assert (input_shape[0] % patch_size == 0) and (
         input_shape[1] % patch_size == 0), "image size must be a multiple of patch_size"
 
+    if last_norm not in ('LN', 'BN'):
+        raise NotImplementedError("last_norm should be in ['LN', 'BN']"
+                                  f"but received {norm}.")
+
     # Normalize image adding rescaling layer
     x = keras.layers.Input(shape=input_shape, name="input")
     y = keras.layers.Rescaling(1 / 127.5, -1, name="Rescale")(x)
@@ -166,7 +180,7 @@ def vit_imagenet(input_shape,
     y = keras.layers.Reshape((y.shape[1] * y.shape[2], hidden_size))(y)
     y = ClassToken(name="ClassToken")(y)
     y = AddPositionEmbs(name="Transformer/PosEmbed")(y)
-    for n in range(num_layers):
+    for n in range(num_blocks):
         y, _ = transformer_block(
             y,
             num_heads=num_heads,
@@ -178,14 +192,23 @@ def vit_imagenet(input_shape,
             softmax=softmax,
             mlp_act=act,
         )
-    y = norm_to_layer(norm)(epsilon=1e-6, name="Transformer/EncoderNorm")(y)
-    y = ExtractToken(token=0, name="ExtractToken")(y)
+
+    # Include classification head
     if include_top:
+        y = norm_to_layer(last_norm)(
+            epsilon=1e-6, name="Transformer/EncoderNorm")(y)
+        y = ExtractToken(token=0, name="ExtractToken")(y)
         y = keras.layers.Dense(classes, name="Head")(y)
     return keras.models.Model(inputs=x, outputs=y, name=name)
 
 
-def vit_ti16(input_shape=(224, 224, 3), classes=1000, norm='LN', softmax='softmax', act='GeLU'):
+def vit_ti16(input_shape=(224, 224, 3),
+             classes=1000,
+             norm='LN',
+             last_norm='LN',
+             softmax='softmax',
+             act='GeLU',
+             include_top=True):
     """Build ViT-Tiny. All arguments passed to vit_imagenet.
 
     Args:
@@ -194,10 +217,15 @@ def vit_ti16(input_shape=(224, 224, 3), classes=1000, norm='LN', softmax='softma
         norm (str, optional): string that values in ['LN', 'GN1', 'BN', 'LMN'] and that allows to
             choose from LayerNormalization, GroupNormalization(groups=1, ...), BatchNormalization
             or LayerMadNormalization layers respectively in the model. Defaults to 'LN'.
+        last_norm (str, optional): string that values in ['LN', 'BN']
+            and that allows to choose from LayerNormalization or
+            BatchNormalization in the classifier network. Defaults to 'LN'.
         softmax (str, optional): string with values in ['softmax', 'softmax2'] that allows to choose
             between softmax and softmax2 in attention block. Defaults to 'softmax'.
         act (str, optional): string that values in ['GeLU', 'ReLUx', 'swish'] and that allows to
             choose from GeLU, ReLUx or swish activation inside MLP. Defaults to 'GeLU'.
+        include_top (bool, optional): whether to include the final classifier network.
+            Defaults to True.
 
     Returns:
         keras.Model: the requested model
@@ -208,18 +236,22 @@ def vit_ti16(input_shape=(224, 224, 3), classes=1000, norm='LN', softmax='softma
         input_shape=input_shape,
         classes=classes,
         norm=norm,
+        last_norm=last_norm,
         act=act,
         softmax=softmax,
+        include_top=include_top,
         **CONFIG_TI
     )
 
 
-def bc_vit_ti16(input_shape=(224, 224, 3), classes=1000):
+def bc_vit_ti16(input_shape=(224, 224, 3), classes=1000, include_top=True):
     """Build ViT-Tiny, changing all LN by LMN, using softmax2 and ReLU8.
 
     Args:
         input_shape (tuple, optional): input shape. Defaults to (224, 224, 3).
         classes (int, optional): number of classes. Defaults to 1000.
+        include_top (bool, optional): whether to include the final classifier network.
+            Defaults to True.
 
     Returns:
         keras.Model: the requested model
@@ -230,8 +262,10 @@ def bc_vit_ti16(input_shape=(224, 224, 3), classes=1000):
         input_shape=input_shape,
         classes=classes,
         norm="LMN",
+        last_norm="BN",
         softmax="softmax2",
         act="ReLU8",
+        include_top=include_top,
         **CONFIG_TI
     )
 
@@ -244,7 +278,7 @@ def bc_vit_ti16_imagenet_pretrained():
         formatted dict.
     """
     model_name = 'bc_vit_ti16_224_quant_config_4bit.h5'
-    file_hash = '5ca9889ca844964224d79fdb7dfb689471258ae6be5db85df2bd86a2640c01b3'
+    file_hash = '8c3623e293e91f19c332eefaa54fb77b88118a3868eeb50e870e51b89857ceb1'
     model_path = fetch_file(BASE_WEIGHT_PATH + model_name,
                             fname=model_name,
                             file_hash=file_hash,
@@ -252,12 +286,14 @@ def bc_vit_ti16_imagenet_pretrained():
     return load_model(model_path)
 
 
-def vit_s16(input_shape=(224, 224, 3), classes=1000):
+def vit_s16(input_shape=(224, 224, 3), classes=1000, include_top=True):
     """Build ViT-Small.
 
     Args:
         input_shape (tuple, optional): input shape. Defaults to (224, 224, 3).
         classes (int, optional): number of classes. Defaults to 1000.
+        include_top (bool, optional): whether to include the final classifier network.
+            Defaults to True.
 
     Returns:
         keras.Model: the requested model
@@ -267,16 +303,19 @@ def vit_s16(input_shape=(224, 224, 3), classes=1000):
         patch_size=16,
         input_shape=input_shape,
         classes=classes,
+        include_top=include_top,
         **CONFIG_S
     )
 
 
-def vit_s32(input_shape=(224, 224, 3), classes=1000):
+def vit_s32(input_shape=(224, 224, 3), classes=1000, include_top=True):
     """Build ViT-Small.
 
     Args:
         input_shape (tuple, optional): input shape. Defaults to (224, 224, 3).
         classes (int, optional): number of classes. Defaults to 1000.
+        include_top (bool, optional): whether to include the final classifier network.
+            Defaults to True.
 
     Returns:
         keras.Model: the requested model
@@ -286,16 +325,19 @@ def vit_s32(input_shape=(224, 224, 3), classes=1000):
         patch_size=32,
         input_shape=input_shape,
         classes=classes,
+        include_top=include_top,
         **CONFIG_S
     )
 
 
-def vit_b16(input_shape=(224, 224, 3), classes=1000):
+def vit_b16(input_shape=(224, 224, 3), classes=1000, include_top=True):
     """Build ViT-B16.
 
     Args:
         input_shape (tuple, optional): input shape. Defaults to (224, 224, 3).
         classes (int, optional): number of classes. Defaults to 1000.
+        include_top (bool, optional): whether to include the final classifier network.
+            Defaults to True.
 
     Returns:
         keras.Model: the requested model
@@ -305,16 +347,19 @@ def vit_b16(input_shape=(224, 224, 3), classes=1000):
         patch_size=16,
         input_shape=input_shape,
         classes=classes,
+        include_top=include_top,
         **CONFIG_B
     )
 
 
-def vit_b32(input_shape=(224, 224, 3), classes=1000):
+def vit_b32(input_shape=(224, 224, 3), classes=1000, include_top=True):
     """Build ViT-B32.
 
     Args:
         input_shape (tuple, optional): input shape. Defaults to (224, 224, 3).
         classes (int, optional): number of classes. Defaults to 1000.
+        include_top (bool, optional): whether to include the final classifier network.
+            Defaults to True.
 
     Returns:
         keras.Model: the requested model
@@ -324,16 +369,19 @@ def vit_b32(input_shape=(224, 224, 3), classes=1000):
         patch_size=32,
         input_shape=input_shape,
         classes=classes,
+        include_top=include_top,
         **CONFIG_B
     )
 
 
-def vit_l16(input_shape=(384, 384, 3), classes=1000):
+def vit_l16(input_shape=(384, 384, 3), classes=1000, include_top=True):
     """Build ViT-L16.
 
     Args:
         input_shape (tuple, optional): input shape. Defaults to (384, 384, 3).
         classes (int, optional): number of classes. Defaults to 1000.
+        include_top (bool, optional): whether to include the final classifier network.
+            Defaults to True.
 
     Returns:
         keras.Model: the requested model
@@ -343,16 +391,19 @@ def vit_l16(input_shape=(384, 384, 3), classes=1000):
         patch_size=16,
         input_shape=input_shape,
         classes=classes,
+        include_top=include_top,
         **CONFIG_L
     )
 
 
-def vit_l32(input_shape=(384, 384, 3), classes=1000):
+def vit_l32(input_shape=(384, 384, 3), classes=1000, include_top=True):
     """Build ViT-L32.
 
     Args:
         input_shape (tuple, optional): input shape. Defaults to (384, 384, 3).
         classes (int, optional): number of classes. Defaults to 1000.
+        include_top (bool, optional): whether to include the final classifier network.
+            Defaults to True.
 
     Returns:
         keras.Model: the requested model
@@ -362,5 +413,6 @@ def vit_l32(input_shape=(384, 384, 3), classes=1000):
         patch_size=32,
         input_shape=input_shape,
         classes=classes,
+        include_top=include_top,
         **CONFIG_L
     )
